import {
  IsOptional,
  IsBoolean,
  IsInt,
  IsEnum,
  ValidateNested,
  ArrayMinSize,
  IsUUID,
  IsArray,
  IsISO8601,
} from 'class-validator';
import { Type } from 'class-transformer';
import { ScheduleItemDto } from './schedule-item.dto';
import { WaypointDto } from './waypoint.dto';
import { HasValidPositionIndexes } from './validators/decorators/has-valid-position-indexes.decorator';
import { Frequency } from '@modules/ad/core/domain/ad.types';
import { IsAfterOrEqual } from './validators/decorators/is-after-or-equal.decorator';
import { HasDay } from './validators/decorators/has-day.decorator';

export class CreateAdRequestDto {
  @IsUUID(4)
  userId: string;

  @IsOptional()
  @IsBoolean()
  driver?: boolean;

  @IsOptional()
  @IsBoolean()
  passenger?: boolean;

  @IsEnum(Frequency)
  @HasDay('schedule', {
    message: 'At least a day is required for a recurrent ad',
  })
  frequency: Frequency;

  @IsISO8601({
    strict: true,
    strictSeparator: true,
  })
  fromDate: string;

  @IsISO8601({
    strict: true,
    strictSeparator: true,
  })
  @IsAfterOrEqual('fromDate', {
    message: 'toDate must be after or equal to fromDate',
  })
  toDate: string;

  @Type(() => ScheduleItemDto)
  @IsArray()
  @ArrayMinSize(1)
  @ValidateNested({ each: true })
  schedule: ScheduleItemDto[];

  @IsOptional()
  @IsInt()
  seatsProposed?: number;

  @IsOptional()
  @IsInt()
  seatsRequested?: number;

  @IsOptional()
  @IsBoolean()
  strict?: boolean;

  @Type(() => WaypointDto)
  @IsArray()
  @ArrayMinSize(2)
  @HasValidPositionIndexes()
  @ValidateNested({ each: true })
  waypoints: WaypointDto[];
}
